@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2013 Sam Trenholme
+/* Copyright (c) 2002-2014 Sam Trenholme
  *
  * TERMS
  *
@@ -29,12 +29,6 @@
 
 #include <stdint.h>
 
-/* Gerrit P. Haase's suggested the following for compiling this on
-   Cygwin */
-#ifdef __CYGWIN__
-#define MSG_WAITALL 0x40
-#endif /* __CYGWIN__ */
-
 /* This is the root server list as of 2007/11/15; the root servers were
  * last changed 2007/11/01 */
 #define ROOT_SERVERS "198.41.0.4"       /* a.root-servers.net (VeriSign) */ \
