@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2014 Sam Trenholme
+/* Copyright (c) 2007-2015 Sam Trenholme
  *
  * TERMS
  *
@@ -224,6 +224,40 @@ void dw_log_dwstr(char *s1, dw_str *s2, int min_log_level) {
 #endif /* MINGW */
 }
 
+/* Log a string followed by the contents of a DwStr object as a series of
+ * decimal integers separated by dots (so IPs in strings look normal) */
+void dw_log_dwstrip(char *s1, dw_str *s2, int min_log_level) {
+        int32_t ll = key_n[DWM_N_verbose_level];
+        int a;
+        if(ll <= 0 || ll < min_log_level) {
+                return;
+        }
+
+#ifndef MINGW
+        printf("%s",s1);
+#else /* MINGW */
+        fprintf(LOG,"%s",s1);
+#endif /* MINGW */
+
+        if(s2 != 0 && s2->str != 0) {
+                for(a=0;a<s2->len;a++) {
+#ifndef MINGW
+                        printf("%d.",*(s2->str + a));
+#else /* MINGW */
+                        fprintf(LOG,"%d.",*(s2->str + a));
+#endif /* MINGW */
+                }
+        }
+
+        /* OK, add a newline */
+
+#ifndef MINGW
+        printf("%s","\n");
+#else /* MINGW */
+        fprintf(LOG,"%s","\n");
+#endif /* MINGW */
+}
+
 /* Log a string followed by the contents of a DwStr object followed by
  * another string */
 void dw_log_dwstr_str(char *s1, dw_str *s2, char *s3, int min_log_level) {
@@ -763,8 +797,8 @@ int32_t get_dns_qid(unsigned char *a, int len, int roy_arends_check) {
         if(roy_arends_check == 2 && (
            /* a[2]: QR, Opcode (4 bits), AA, TC, and RD */
            ((a[2] & 0xf9)) != 0x01 ||
-           /* a[3]: RA, Z (3 bits), RCODE (4 bits) */
-           ((a[3] & 0x7f)) != 0x00 ||
+           /* a[3]: RA, Z (1 bit), AD & CD (2 bits, RFC2535), RCODE (4 bits) */
+           ((a[3] & 0x4f)) != 0x00 ||
            /* a[4] and a[5]: QDCOUNT */
            a[4] != 0 || a[5] != 1 ||
            /* a[6], a[7]: ANCOUNT */
