@@ -1,4 +1,5 @@
-/* Copyright (c) 2007-2010 Sam Trenholme and Jean-Jacques Sarton
+/* Copyright (c) 2007-2012 Sam Trenholme
+ * IPv6 code contributed by Jean-Jacques Sarton in 2007
  *
  * TERMS
  *
@@ -62,7 +63,7 @@ extern void windows_socket_start();
  * This memory is never freed once allocated, because this data is always
  * used by the program */
 void malloc_tcp_pend() {
-        tcp_pend = malloc((max_tcp_procs + 1) * sizeof(tcp_pend_T));
+        tcp_pend = dw_malloc((max_tcp_procs + 1) * sizeof(tcp_pend_T));
         if(tcp_pend == 0) {
                 dw_alog_3strings("Fatal: Could not allocate tcp_pend","","");
                 exit(1);
@@ -176,7 +177,7 @@ void local_tcp_accept(SOCKET s) {
          * the information we want, then we went to send the reply back
          * over TCP */
         init_tcp_pend(b);
-        tcp_pend[b].buffer = malloc(3); /* To put the two bytes we want */
+        tcp_pend[b].buffer = dw_malloc(3); /* To put the two bytes we want */
         if(tcp_pend[b].buffer == 0) {
                 closesocket(local);
                 reset_tcp_pend(b);
@@ -196,7 +197,7 @@ void tcp_get_wanted(int b) {
         int toget = 0;
         toget = tcp_pend[b].wanted - tcp_pend[b].got;
         if(toget > 0 && tcp_pend[b].state == 0) {
-                buffer = malloc(toget + 1);
+                buffer = dw_malloc(toget + 1);
                 if(buffer == 0) {
                         return;
                 }
@@ -239,7 +240,7 @@ void tcp_process_data(int b) {
                         return;
                 }
                 tcp_pend[b].wanted = wanted;
-                tcp_pend[b].buffer = malloc(wanted + 1);
+                tcp_pend[b].buffer = dw_malloc(wanted + 1);
                 tcp_pend[b].die = get_time() +
                         ((int64_t)timeout_seconds_tcp << 8);
         } else if(tcp_pend[b].wanted >= 12) {
@@ -298,7 +299,7 @@ void tcp_return_reply(int b, char *packet, int len) {
                 tcp_pend[b].buffer = 0;
         }
         tcp_pend[b].state = 2; /* Send TCP reply back to client */
-        tcp_pend[b].buffer = malloc(len + 3);
+        tcp_pend[b].buffer = dw_malloc(len + 3);
         /* 2-byte length header */
         tcp_pend[b].buffer[0] = ((len & 0xff00) >> 8);
         tcp_pend[b].buffer[1] = (len & 0xff);
@@ -378,7 +379,7 @@ void tcp_truncated_retry(int b, dw_str *query, int id, int udp_id, int is_up) {
         if(tmp == 0) {
                 goto catch_tcp_truncated_retry;
         }
-        tcp_pend[b].buffer = malloc(tmp->len + 3);
+        tcp_pend[b].buffer = dw_malloc(tmp->len + 3);
         if(tcp_pend[b].buffer == 0) {
                 goto catch_tcp_truncated_retry;
         }
@@ -453,7 +454,7 @@ void tcp_prepare_upstream_len(int b) {
         if(tcp_pend[b].state != 4) {
                 return;
         }
-        tcp_pend[b].buffer = malloc(3);
+        tcp_pend[b].buffer = dw_malloc(3);
         if(tcp_pend[b].buffer == 0) {
                 closesocket(tcp_pend[b].local);
                 closesocket(tcp_pend[b].upstream);
@@ -492,7 +493,7 @@ void tcp_get_upstream_len(int b) {
                 free(tcp_pend[b].buffer);
                 tcp_pend[b].buffer = 0;
                 tcp_pend[b].wanted = wanted;
-                tcp_pend[b].buffer = malloc(wanted + 3);
+                tcp_pend[b].buffer = dw_malloc(wanted + 3);
                 tcp_pend[b].die = get_time() +
                         ((int64_t)timeout_seconds_tcp << 8);
                 tcp_pend[b].state = 6;
