@@ -1,4 +1,4 @@
-/* Copyright (c) 2007-2014 Sam Trenholme and others
+/* Copyright (c) 2007-2015 Sam Trenholme and others
  *
  * TERMS
  *
@@ -47,7 +47,7 @@ extern remote_T *rem;
 dw_hash *inflight;
 
 /* The numeric mararc parameters */
-extern int maxprocs;
+extern int_fast32_t maxprocs;
 extern int timeout_seconds;
 extern int dns_port;
 extern int upstream_port;
@@ -404,6 +404,10 @@ int forward_local_udp_packet(SOCKET sock, int32_t local_id,
 #endif /* INFLIGHT_VERBOSE */
         }
 
+        if(rem[b].num_locals == 0) {
+                reset_rem(b);
+                return -1;
+        }
         rem[b].local[rem[b].num_locals - 1] = dw_malloc(sizeof(local_T));
         if(rem[b].local[rem[b].num_locals - 1] != 0) {
                 rem[b].local[rem[b].num_locals - 1]->orig_query = 0;
