@@ -61,7 +61,7 @@ js_string *js_create_DEBUG(unsigned int max_count, unsigned int unit_size,
     new->encoding = JS_BINARY;
     /* Allocate memory for character string, return on error */
     /* The 3 is a security margin */
-    if((new->string = js_alloc_DEBUG(max_count + 3,unit_size,whence)) 
+    if((new->string = js_alloc_DEBUG(max_count + 3,unit_size,whence))
        == (void *)0) {
         js_dealloc(new);
         return (js_string *)0;
@@ -294,8 +294,9 @@ int js_issame(js_string *js1, js_string *js2) {
     if(js1->unit_count != js2->unit_count)
         return 0;
     /* They both have to use the same encoding */
-    if(js1->encoding != js2->encoding)
-        return 0;
+    /*if(js1->encoding != js2->encoding)
+        return 0; */ /* This requirement disabled because this just
+                        causes annoyances */
 
     max = js1->unit_count;
 
@@ -877,6 +878,42 @@ int js_qprepend(char *toprepend, js_string *target) {
 
     }
 
+/* js_qissame: "Quick" version of js_issame routine
+   input: NULL-terminated string to look for, pointer to js object
+   output: JS_ERROR on failure, JS_SUCCESS on success
+*/
+
+int js_qissame(char *expression, js_string *target) {
+
+    js_string *temp = 0;
+    int return_value;
+
+    /* Sanity test */
+    if(js_has_sanity(target) == JS_ERROR)
+        return JS_ERROR;
+
+    /* Allocate memory for the appending string if necessary */
+    if(temp == 0)
+        /* We really should make 1024 here strlen(toappend)
+           That would necessitate moving this to JsStrOS.c */
+        if((temp = js_create(1024,target->unit_size)) == 0)
+            return JS_ERROR;
+
+    if(js_qstr2js(temp,expression) == JS_ERROR) {
+        js_destroy(temp);
+        return JS_ERROR;
+        }
+
+    temp->unit_size = target->unit_size;
+    temp->encoding = target->encoding;
+    return_value = js_issame(temp,target);
+
+    js_destroy(temp);
+
+    return return_value;
+
+    }
+
 /* js_qfgrep: "Quick" version of js_fgrep routine
    input: NULL-terminated string to look for, pointer to js object
    output: JS_ERROR on failure, JS_SUCCESS on success
@@ -1047,7 +1084,7 @@ int js_substr_append(js_string *source, js_string *dest, int offset,
     js_string *temp;
     if((temp = js_create(length + 2,1)) == 0) {
         return JS_ERROR;
-	}
+        }
 
     if(source->unit_count < offset + length) {
        js_destroy(temp);
@@ -1061,8 +1098,8 @@ int js_substr_append(js_string *source, js_string *dest, int offset,
 
     if(js_append(temp,dest) == JS_ERROR) {
         js_destroy(temp);
-	return JS_ERROR;
-	}
+        return JS_ERROR;
+        }
 
     js_destroy(temp);
     return JS_SUCCESS;
