@@ -1,13 +1,26 @@
-/* Public domain code by Sam Trenholme 2000-2002 */
-/* Routines that interface between the string library and the underlying
-   OS */
+/* Copyright (c) 2002-2006 Sam Trenholme
+ *
+ * TERMS
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * This software is provided 'as is' with no guarantees of correctness or
+ * fitness for purpose.
+ */
 
 /* Headers for the underlying OS calls */
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <sys/stat.h>
-#include <sys/types.h>
 #include <fcntl.h>
 #include <sys/file.h>
 #include <limits.h>
@@ -31,7 +44,7 @@ typedef struct lt_hash_spot {
 int lt_hash_initialized = 0;
 lt_hash_spot lt_hash[HASH_SIZE];
 int total_allocated_memory = 0;
-#define THREADS 
+#define THREADS
 #endif /* DEBUG */
 #ifdef THREADS
 pthread_mutex_t alloc_lock = PTHREAD_MUTEX_INITIALIZER;
@@ -43,56 +56,59 @@ pthread_mutex_t alloc_lock = PTHREAD_MUTEX_INITIALIZER;
 
 void *js_alloc(int unit_count, int unit_size) {
     void *data;
-#ifdef DEBUG 
+#ifdef DEBUG
     unsigned int hash_index,counter;
     lt_hash_spot *new,*point;
 #endif /* DEBUG */
+    /* Sanity check: Never allow this; makes C act buggy */
+    if(unit_size == 0 || unit_count == 0)
+        return 0;
     data = (void *)malloc(unit_count * unit_size);
 #ifdef DEBUG
-    /* If debug is set, we have a 64k element hash table which uses the low 
-       16 bits of the memory allocated as the hash index.  Each element in 
+    /* If debug is set, we have a 64k element hash table which uses the low
+       16 bits of the memory allocated as the hash index.  Each element in
        the hash table is a linked list, which has the address and number of
        bytes allocated for each piece of allocated memory */
 
-    (int)hash_index = (void *)data;
+    hash_index = (int)data;
     hash_index %= HASH_SIZE;
 #ifdef THREADS
     pthread_mutex_lock(&alloc_lock);
 #endif /* THREADS */
     /* Initialize the lt hash the first time we run this routine */
     if(lt_hash_initialized == 0) {
-        for(counter = 0; counter < HASH_SIZE; counter++) 
-	    lt_hash[counter].allocated_size = 0;
+        for(counter = 0; counter < HASH_SIZE; counter++)
+            lt_hash[counter].allocated_size = 0;
         lt_hash_initialized = 1;
         }
     /* If the hash pointer is empty, use that */
     if(lt_hash[hash_index].allocated_size == 0) {
         lt_hash[hash_index].allocated_size = unit_count * unit_size;
-	lt_hash[hash_index].allocated_memory = data;
-	lt_hash[hash_index].de_donde = 0;
+        lt_hash[hash_index].allocated_memory = data;
+        lt_hash[hash_index].de_donde = 0;
         lt_hash[hash_index].next = 0;
         }
     /* Otherwise, go down the linked list, place the new node at the
        end of the branch in question */
     else {
         new = malloc(sizeof(lt_hash_spot));
-	if(new == NULL) {
-	    printf("Aieeee, can not allocate memeory for lt_hash_spot!\n");
-	    exit(1);
-	    }
+        if(new == NULL) {
+            printf("Aieeee, can not allocate memeory for lt_hash_spot!\n");
+            exit(1);
+            }
         new->allocated_size = unit_count * unit_size;
         new->next = 0;
-	new->allocated_memory = data;
-	new->de_donde = 0;
+        new->allocated_memory = data;
+        new->de_donde = 0;
         point = lt_hash[hash_index].next;
-	if(point == 0) {
-	    lt_hash[hash_index].next = new;
-	    }
-	else {
-	    while(point->next != 0) 
-	        point = point->next;
+        if(point == 0) {
+            lt_hash[hash_index].next = new;
+            }
+        else {
+            while(point->next != 0)
+                point = point->next;
             point->next = new;
-	    }
+            }
         }
     total_allocated_memory += unit_count * unit_size;
 #ifdef THREADS
@@ -117,64 +133,64 @@ void *js_alloc(int unit_count, int unit_size) {
 
 void *js_alloc_DEBUG(int unit_count, int unit_size, char *whence) {
     void *data;
-#ifdef DEBUG 
+#ifdef DEBUG
     unsigned int hash_index,counter;
     lt_hash_spot *new,*point;
     char *why;
 #endif /* DEBUG */
     data = (void *)malloc(unit_count * unit_size);
 #ifdef DEBUG
-    /* If debug is set, we have a 64k element hash table which uses the low 
-       16 bits of the memory allocated as the hash index.  Each element in 
+    /* If debug is set, we have a 64k element hash table which uses the low
+       16 bits of the memory allocated as the hash index.  Each element in
        the hash table is a linked list, which has the address and number of
        bytes allocated for each piece of allocated memory */
 
-    (int)hash_index = (void *)data;
+    hash_index = (int)data;
     hash_index %= HASH_SIZE;
     /* Set up the string used to tell people why this memory was
        allocated */
     if((why = malloc(strlen(whence) + 2)) == 0) {
         printf("Aieeee, mem alloc problem\n");
-	exit(64);
-	}
+        exit(64);
+        }
     strncpy(why,whence,strlen(whence) + 1);
 #ifdef THREADS
     pthread_mutex_lock(&alloc_lock);
 #endif /* THREADS */
     /* Initialize the lt hash the first time we run this routine */
     if(lt_hash_initialized == 0) {
-        for(counter = 0; counter < HASH_SIZE; counter++) 
-	    lt_hash[counter].allocated_size =0;
+        for(counter = 0; counter < HASH_SIZE; counter++)
+            lt_hash[counter].allocated_size =0;
         lt_hash_initialized = 1;
         }
     /* If the hash pointer is empty, use that */
     if(lt_hash[hash_index].allocated_size == 0) {
         lt_hash[hash_index].allocated_size = unit_count * unit_size;
-	lt_hash[hash_index].allocated_memory = data;
-	lt_hash[hash_index].de_donde = why;
+        lt_hash[hash_index].allocated_memory = data;
+        lt_hash[hash_index].de_donde = why;
         lt_hash[hash_index].next = 0;
         }
     /* Otherwise, go down the linked list, place the new node at the
        end of the branch in question */
     else {
         new = malloc(sizeof(lt_hash_spot));
-	if(new == NULL) {
-	    printf("Aieeee, can not allocate memeory for lt_hash_spot!\n");
-	    exit(1);
-	    }
+        if(new == NULL) {
+            printf("Aieeee, can not allocate memeory for lt_hash_spot!\n");
+            exit(1);
+            }
         new->allocated_size = unit_count * unit_size;
         new->next = 0;
-	new->allocated_memory = data;
-	new->de_donde = why;
+        new->allocated_memory = data;
+        new->de_donde = why;
         point = lt_hash[hash_index].next;
-	if(point == 0) {
-	    lt_hash[hash_index].next = new;
-	    }
-	else {
-	    while(point->next != 0) 
-	        point = point->next;
+        if(point == 0) {
+            lt_hash[hash_index].next = new;
+            }
+        else {
+            while(point->next != 0)
+                point = point->next;
             point->next = new;
-	    }
+            }
         }
     total_allocated_memory += unit_count * unit_size;
 #ifdef THREADS
@@ -201,64 +217,66 @@ int js_dealloc(void *pointer) {
     lt_hash_spot *lpoint,*llast;
     unsigned int hash_index;
 #endif /* DEBUG */
+    if(pointer == 0) /* We don't bother with already dealloced memory */
+            return 1;
 #ifdef DEBUG
-    /* We need to find out the number of bytes that we have allocated 
+    /* We need to find out the number of bytes that we have allocated
        for this given address; subtract that from the total amount of
-       memory used, then removed the node which keeps track of the allocated 
+       memory used, then removed the node which keeps track of the allocated
        memory in question.  */
 #ifdef THREADS
     pthread_mutex_lock(&alloc_lock);
 #endif /* THREADS */
-    (int)hash_index = (void *)pointer;
+    hash_index = (int)pointer;
     hash_index %= HASH_SIZE;
     if(lt_hash[hash_index].allocated_memory == pointer) {
         lpoint = lt_hash[hash_index].next;
         total_allocated_memory -= lt_hash[hash_index].allocated_size;
-	if(lpoint == 0) {
-	    lt_hash[hash_index].allocated_size = 0;
-	    lt_hash[hash_index].allocated_memory = 0;
-	    if(lt_hash[hash_index].de_donde != 0)
-	        free(lt_hash[hash_index].de_donde);
+        if(lpoint == 0) {
+            lt_hash[hash_index].allocated_size = 0;
+            lt_hash[hash_index].allocated_memory = 0;
+            if(lt_hash[hash_index].de_donde != 0)
+                free(lt_hash[hash_index].de_donde);
             lt_hash[hash_index].de_donde = 0;
-	    }
+            }
         else {
-	    lt_hash[hash_index].allocated_size = lpoint->allocated_size;
-	    lt_hash[hash_index].allocated_memory = lpoint->allocated_memory;
-	    lt_hash[hash_index].next = lpoint->next;
-	    if(lt_hash[hash_index].de_donde != 0)
-	        free(lt_hash[hash_index].de_donde);
+            lt_hash[hash_index].allocated_size = lpoint->allocated_size;
+            lt_hash[hash_index].allocated_memory = lpoint->allocated_memory;
+            lt_hash[hash_index].next = lpoint->next;
+            if(lt_hash[hash_index].de_donde != 0)
+                free(lt_hash[hash_index].de_donde);
             lt_hash[hash_index].de_donde = lpoint->de_donde;
-	    free(lpoint);
-	    }
+            free(lpoint);
+            }
         }
     else {
         lpoint = lt_hash[hash_index].next;
-	llast = 0;
-	while(lpoint->allocated_memory != pointer) {
-	    llast = lpoint;
-	    lpoint = lpoint->next;
-	    if(lpoint == 0) {
-	        printf("Fatal: lpoint is 0; should not be\n");
-		printf("Debugging info: hash_index %x\n",hash_index);
-		lpoint = lt_hash[hash_index].next;
-		while(lpoint != 0) {
-		    printf("lpoint: %p\n",lpoint);
-		    lpoint = lpoint->next;
-		    }
+        llast = 0;
+        while(lpoint->allocated_memory != pointer) {
+            llast = lpoint;
+            lpoint = lpoint->next;
+            if(lpoint == 0) {
+                printf("Fatal: lpoint is 0; should not be\n");
+                printf("Debugging info: hash_index %x\n",hash_index);
+                lpoint = lt_hash[hash_index].next;
+                while(lpoint != 0) {
+                    printf("lpoint: %p\n",lpoint);
+                    lpoint = lpoint->next;
+                    }
                 exit(1);
-		}
-	    }
+                }
+            }
         total_allocated_memory -= lpoint->allocated_size;
-	if(llast == 0) {
-	    lt_hash[hash_index].next = lpoint->next;
-	    }
+        if(llast == 0) {
+            lt_hash[hash_index].next = lpoint->next;
+            }
         else {
-	    llast->next = lpoint->next;
-	    }
+            llast->next = lpoint->next;
+            }
         if(lpoint->de_donde != 0)
-	    free(lpoint->de_donde);
+            free(lpoint->de_donde);
         free(lpoint);
-	}
+        }
 #ifdef THREADS
     pthread_mutex_unlock(&alloc_lock);
 #endif /* THREADS */
@@ -276,11 +294,11 @@ int js_show_leaks() {
     for(counter = 0; counter < HASH_SIZE; counter++) {
         point = &lt_hash[counter];
         while(point != 0 && point->allocated_memory != 0) {
-	    if(point->de_donde != 0) {
+            if(point->de_donde != 0) {
                 printf("%s: %d bytes at %p\n",point->de_donde,
-		       point->allocated_size,point->allocated_memory);
-		total += point->allocated_size;
-		}
+                       point->allocated_size,point->allocated_memory);
+                total += point->allocated_size;
+                }
             point = point->next;
             }
         }
@@ -298,8 +316,8 @@ int js_show_leaks() {
 /* Routine which tells us how much memory we have allocated.
    Input: None
    Output: The number of bytes allocated by js_alloc routines; 0 if
-           it can not be determine (we do not have DEBUG set to keep 
-	   track of it)
+           it can not be determine (we do not have DEBUG set to keep
+           track of it)
 */
 
 int js_tell_memory_allocated() {
@@ -328,6 +346,80 @@ int js_show_stdout(js_string *js) {
     return 1;
     }
 
+/* show_esc_stdout: Display a csv1-compatible backslash escaped
+                    version of a given js_string object on standard output
+   input: Pointer to js_string object
+   output: -1 on failure, 1 on success */
+int show_esc_stdout(js_string *js) {
+    int counter = 0;
+    unsigned char this;
+
+    if(js_has_sanity(js) < 0)
+        return -1;
+
+    if(js->unit_size != 1) /* There is a bug which changes this while keeping
+                              the sanity */
+        return -1;
+    while(counter < js->unit_size * js->unit_count) {
+        this = *(js->string + counter);
+        if(this < 32 || this > 126) {
+            printf("\\%03o",this);
+            }
+        else if(this == '\\' || this == '%') {
+            printf("\\%c",this);
+            }
+        else {
+            putc(*(js->string + counter),stdout);
+            }
+        counter++;
+        }
+
+    return 1;
+    }
+
+/* safe_esc_stdout: Display a safe csv2-compatible backslash escaped
+                    version of a dlabel; this will make an
+                    hex escape sequence anything
+                    that is not [A-Za-z0-9\-\_]
+   input: Pointer to js_string object
+   output: -1 on failure, 1 on success */
+int safe_esc_stdout(js_string *js) {
+    int counter = 0;
+    unsigned char this;
+
+    if(js_has_sanity(js) < 0)
+        return -1;
+
+    if(js->unit_size != 1) /* There is a bug which changes this while keeping
+                              the sanity */
+        return -1;
+    while(counter < js->unit_size * js->unit_count) {
+        this = *(js->string + counter);
+        /* A lot of the fetchzone code makes the first character a
+         * space, which needs to be faithfully copied */
+        if(this == ' ' && counter == 0) {
+                printf(" ");
+        }
+        else if((this >= 'A' && this <= 'Z') || (this >= 'a' && this <= 'z')
+           || this == '-' || this == '_' || this > 127 /* UTF-8 */
+           || this == '.' || this == '@' || (this >= '0' && this <= '9')) {
+            printf("%c",this);
+            }
+        /* We show a star only if it is the first character and the second
+         * character is a '.' character */
+        else if(this == '*' && counter == 0 && js->unit_count > 2 &&
+                *(js->string + 1) == '.') {
+            printf("%c",this);
+            }
+        else {
+            printf("\\x%02x",this);
+            }
+        counter++;
+        }
+
+    return 1;
+    }
+
 /* js_getline_stdin: Get a line from the standard input
    input: To to js_string object to put stdin contents in
    output: JS_ERROR on error, JS_SUCCESS on success.
@@ -487,8 +579,17 @@ int js_read(js_file *desc, js_string *js, int count) {
         return JS_ERROR;
     if(count % js->unit_size != 0)
         return JS_ERROR;
-    if(count < 0 || count > SSIZE_MAX)
+#ifndef MINGW32
+    if(count < 0 || count > 1048576)
         return JS_ERROR;
+#if (SSIZE_MAX < 1048576)
+    if(count > SSIZE_MAX)
+        return JS_ERROR;
+#endif /* SSIZE_MAX small */
+#else
+    if(count < 0 || count > 1024)
+        return JS_ERROR;
+#endif
 
     if(count > js->unit_size * js->max_count)
         return JS_ERROR;
@@ -526,8 +627,17 @@ int js_write(js_file *desc, js_string *js) {
         return JS_ERROR;
 
     to_write = js->unit_size * js->unit_count;
-    if(to_write < 0 || to_write > SSIZE_MAX)
+#ifndef MINGW32
+    if(to_write < 0 || to_write > 1048576)
         return JS_ERROR;
+#if (SSIZE_MAX < 1048576)
+    if(to_write > SSIZE_MAX)
+        return JS_ERROR;
+#endif /* SSIZE_MAX small */
+#else
+    if(to_write < 0 || to_write > 1024)
+        return JS_ERROR;
+#endif
 
     value = write(desc->file_desc,js->string,to_write);
 
@@ -745,6 +855,9 @@ int js_buf_getline(js_file *desc, js_string *js) {
    output: JS_ERROR on failure, JS_SUCCESS on success
    note: it's here because of strlen call */
 int js_qstr2js(js_string *js, char *string) {
+    if(js == 0) {
+        return JS_ERROR;
+        }
     return js_str2js(js,string,strlen(string),js->unit_size);
     }
 
@@ -752,10 +865,10 @@ int js_qstr2js(js_string *js, char *string) {
                  in big-endian format, where the js->unit_size is one.
    input: pointer to js_string object, number to add
    output: JS_ERROR on error, JS_SUCCESS on success
-   (This is in OS because of the dependency on u_int32_t
+   (This is in OS because of the dependency on uint32_t
 */
 
-int js_adduint32(js_string *js, u_int32_t number) {
+int js_adduint32(js_string *js, uint32_t number) {
 
     /* sanity checks */
     if(js_has_sanity(js) == JS_ERROR)
@@ -782,12 +895,12 @@ int js_adduint32(js_string *js, u_int32_t number) {
           of string (0 is beginning of string, 1 second byte, etc.)
    output: JS_ERROR on error, value of uint32 on success
            (Hack: 0xffffffff is the same as -1 in comparisons)
-   (This is in OS because of the dependency on u_int32_t)
+   (This is in OS because of the dependency on uint32_t)
 */
 
-u_int32_t js_readuint32(js_string *js, unsigned int offset) {
+uint32_t js_readuint32(js_string *js, unsigned int offset) {
 
-    u_int32_t ret;
+    uint32_t ret;
     /* sanity checks */
     if(js_has_sanity(js) == JS_ERROR)
         return 0xffffffff;
@@ -809,4 +922,22 @@ u_int32_t js_readuint32(js_string *js, unsigned int offset) {
 
     }
 
+/* js_strnlen: Determine the length of a null-terminated string, up
+ * to the length determined by the limit.  This is here because strnlen
+ * is, alas, not a portable string library call.  This call is originally
+ * by Matthew T. Russotto */
+int js_strnlen(char *s, uint32_t limit) {
+        uint32_t len;
+        if(limit > 2147483600 || limit < 0) { /* Just under 2 ** 31 */
+                return JS_ERROR;
+        }
+        if(s == NULL) {
+                return JS_ERROR;
+        }
+        len = 0;
+        while(len < limit && (*s++)) {
+                len++;
+        }
+        return len;
+}
 
