@@ -1,7 +1,7 @@
 /* Function to remove a js_string object from a mhash structure
    input: Hash to fondle, key
    output: JS_SUCCESS or JS_ERROR, depending on success/failure */
- 
+
 int mhash_undef_js(mhash *hash, js_string *key);
 
 /* Function to resize a hash table.
@@ -9,7 +9,7 @@ int mhash_undef_js(mhash *hash, js_string *key);
           new hash table
    output: JS_SUCCESS on success, JS_ERROR on error
 */
- 
+
 int mhash_resize(mhash *hash,int new_bits);
 
 /* Function to, if needed, automatically grow a hash table
@@ -17,7 +17,7 @@ int mhash_resize(mhash *hash,int new_bits);
    Output: JS_ERROR if something bad happened, 1 if the table did not grow,
            2 if the table grew.
 */
- 
+
 int mhash_autogrow(mhash *hash);
 
 /* Convert a hash offset to a pointer to js data for the key
@@ -39,3 +39,5 @@ js_string *mhash_offset2js(mhash *hash, mhash_offset offset);
 */
 void *mhash_undef(mhash *hash, js_string *key);
 
+/* Read four bytes from a filename and use that as a secret add constant */
+int mhash_set_add_constant(char *filename);
