@@ -22,48 +22,15 @@
 #include "../libs/MaraHash.h"
 #include "ParseCsv1_en.h"
 #include <stdlib.h>
+#ifndef MINGW32
 #include <pwd.h>
+#endif
 #include <sys/types.h>
 #include <stdio.h>
 
 /* Function prototypes */
 #include "../dns/functions_dns.h"
-extern int mara_goodjs();
-extern int js_space_chars();
-extern int ddip_2_ip();
-extern int js_changebyte();
-extern int js_numbers();
-
-/* show_esc_stdout: Display a csv1-compatible backslash escaped
-                    version of a given js_string object on standard output
-   input: Pointer to js_string object
-   output: -1 on failure, 1 on success */
-int show_esc_stdout(js_string *js) {
-    int counter = 0;
-    unsigned char this;
-
-    if(js_has_sanity(js) < 0)
-        return -1;
-
-    if(js->unit_size != 1) /* There is a bug which changes this while keeping
-                              the sanity */
-        return -1;
-    while(counter < js->unit_size * js->unit_count) {
-        this = *(js->string + counter);
-        if(this < 32 || this > 126) {
-            printf("\\%03o",this);
-            }
-        else if(this == '\\' || this == '%') {
-            printf("\\%c",this);
-            }
-        else {
-            putc(*(js->string + counter),stdout);
-            }
-        counter++;
-        }
-
-    return 1;
-    }
+#include "functions_parse.h"
 
 /* Parse a single line of a csv1 data file
    input: pointer to line of data, place to put the domain name (with
@@ -142,11 +109,24 @@ int parse_csv1_line(js_string *line, js_string *name, js_string *data,
         goto clean;
     if(js_space_chars(blankq) == JS_ERROR)
         goto clean;
-    /* use starq as tempory string to append newlines to blankq */
+
+    /* use starq as temporary string to append newlines to blankq */
     if(js_newline_chars(starq) == JS_ERROR)
         goto clean;
     if(js_append(starq,blankq) == JS_ERROR)
         goto clean;
+
+    /* While starq has a newline in it, lets make sure the line in
+       question ends with a newline character */
+    tempp = js_match(starq,line);
+    if(tempp == JS_ERROR)
+        goto clean;
+    if(tempp == -2) {
+        js_qstr2js(name,"ERROR");
+        js_qstr2js(data,"No newline found in line");
+        goto error;
+        }
+
     /* Now, give starq its proper value */
     if(js_str2js(starq,&star,1,1) == JS_ERROR)
         goto clean;
@@ -186,7 +166,7 @@ int parse_csv1_line(js_string *line, js_string *name, js_string *data,
     /* OK, copy over the data that extends from the beginning of the
        line to the first pipe */
     if(js_substr(line,name,0,pipep) == JS_ERROR)
-	goto clean;
+        goto clean;
     /* Try to make that string a RFC1035 domain query */
     tempp = hname_2rfc1035(name);
     if(tempp == JS_ERROR) {
@@ -430,7 +410,7 @@ int parse_csv1_line(js_string *line, js_string *name, js_string *data,
     error:
         js_destroy(pipeq); js_destroy(hashq); js_destroy(starq);
         js_destroy(blankq); js_destroy(field);
-        return -1;
+        return -2;
     cero:
         js_destroy(pipeq); js_destroy(hashq); js_destroy(starq);
         js_destroy(blankq); js_destroy(field);
@@ -486,7 +466,7 @@ int ddip_2_ip(js_string *ddip, js_string *ip, int offset) {
             return JS_ERROR;
             }
         /* Place numbers in that string */
-        if(js_numbers(numdotq) == JS_ERROR) {
+        if(js_numbers(numdotq) == 0) {
             js_destroy(numdotq);
             js_destroy(dotq);
             numdotq = 0;
