@@ -1,4 +1,4 @@
-/* Copyright (c) 2002,2003 Sam Trenholme
+/* Copyright (c) 2002-2010 Sam Trenholme
  *
  * TERMS
  *
@@ -30,9 +30,13 @@
 #include <time.h>
 #include <sys/types.h>
 #include <sys/time.h>
+#ifndef MINGW32
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#else
+#include <winsock2.h>
+#endif
 #include "../MaraDns.h"
 /* All of the labels */
 #include "askmara_labels_en.h"
@@ -41,13 +45,25 @@
 #include "../rng/rng-api-fst.h"
 /* END RNG USING CODE */
 
-int verbose_mode = 0; /* Whether to have short or verbose output; 
+#include "../libs/MaraHash.h"
+#include "../dns/functions_dns.h"
+#include "../parse/functions_parse.h"
+
+int verbose_mode = 0; /* Whether to have short or verbose output;
                          1: verbose output; any other value: short output */
 int timeout = 21;
 
+unsigned short dns_port = DNS_PORT;
+
 /* So -Wall doesn't gag */
 extern int decomp_init();
 
+/* Defining finctions which are part of askmara-tcp.c */
+int verbose_output(js_string *uindata);
+int csv1_compatible_output(js_string *uindata, js_string *query);
+int out_answer(js_string *uindata,int *place);
+int csv1_answer(js_string *uindata, int *place, js_string *query);
+
 /* Octal escape any non-ASCII sequences in the askmara output; almost idential
    to show_esc_stdout */
 int escape_stdout(js_string *js) {
@@ -62,12 +78,12 @@ int escape_stdout(js_string *js) {
 
     while(counter < js->unit_count) {
         this = *(js->string + counter);
-	if(this < 32 || this > 126) 
-	    printf("\\%03o",this);
-        else if(this == '\\') 
-	    printf("\\%c",this);
+        if(this < 32 || this > 126)
+            printf("\\%03o",this);
+        else if(this == '\\')
+            printf("\\%c",this);
         else
-	    printf("%c",this);
+            printf("%c",this);
         counter++;
         }
     return 1;
@@ -79,12 +95,11 @@ int escape_stdout(js_string *js) {
 u_int16_t gen_id(char *hostname) {
     /* BEGIN RNG USING CODE */
     /* May as well bring out the bug guns (e.g. our secure RNG) */
-    BYTE r_inBlock[17],r_outBlock[17],r_binKey[17];
-    BYTE r_keyMaterial[320];
+    MARA_BYTE r_inBlock[17],r_outBlock[17],r_binKey[17];
     keyInstance r_keyInst;
     cipherInstance r_cipherInst;
     unsigned char crypto_key[34];
-    int desc, len, counter;
+    int counter;
     char *mc1, *mc2;
     /* END RNG USING CODE */
     int ret = 4321;
@@ -95,34 +110,34 @@ u_int16_t gen_id(char *hostname) {
 
     /* Initialize the keys, including the "binKey" (is this used?) */
     memset(r_binKey,'0',16);
-    mc1 = "ñ";
+    mc1 = "";
     mc2 = mc1;
-    /* Set the crypto key to 'ññññññññññññññññ' (UTF-8 encoding) */
+    /* Set the crypto key to '' (UTF-8 encoding) */
     for(counter = 0; counter < 16; counter++) {
         *(crypto_key + counter) = *mc2;
-	mc2++;
-	if(*mc2 == 0) 
-	   mc2 = mc1;
+        mc2++;
+        if(*mc2 == 0)
+           mc2 = mc1;
         }
-    /*memset(crypto_key,'ñ',16);*/
+    /*memset(crypto_key,'',16);*/
     /* Make the first 16 characters of the hostname the key (padded by the
-       ñ character--do you see a n~ there?) */
-    strncpy(crypto_key,hostname,16);
-    /* Set the plaintext block to 'ññññññññññññññññ' (UTF-8 encoding) */
+        character--do you see a n~ there?) */
+    strncpy((char *)crypto_key,hostname,16);
+    /* Set the plaintext block to '' (UTF-8 encoding) */
     mc2 = mc1;
     for(counter = 0; counter < 16; counter++) {
         *(r_inBlock + counter) = *mc2;
-	mc2++;
-	if(*mc2 == 0) 
-	   mc2 = mc1;
+        mc2++;
+        if(*mc2 == 0)
+           mc2 = mc1;
         }
-    /*memset(r_inBlock,'ñ',16);*/
+    /*memset(r_inBlock,'',16);*/
     /* If the hostname is longer than 16 characters, make the plaintext
        the longer part of the hostname */
     if(strlen(hostname) > 16)
-    strncpy(r_inBlock,hostname+16,16);
+    strncpy((char *)r_inBlock,hostname+16,16);
 
-    if(makeKey(&r_keyInst, DIR_ENCRYPT, 128, crypto_key) != 1)
+    if(makeKey(&r_keyInst, DIR_ENCRYPT, 128, (char *)crypto_key) != 1)
         return JS_ERROR;
     if(cipherInit(&r_cipherInst, MODE_ECB, NULL) != 1)
         return JS_ERROR;
@@ -141,21 +156,20 @@ int harderror(char *msg) {
     exit(1);
     }
 
-main(int argc, char **argv) {
+int main(int argc, char **argv) {
     char *server_address = NULL;
+#if defined IPV6 && ! defined __CYGWIN__
+    struct sockaddr_in6 server;
+#else
     struct sockaddr_in server;
+#endif
     int len_inet; /* Length */
     int s; /* Socket */
-    int q; /* Used for binding, etc */
     js_string *outdata; /* Outgoing data */
     js_string *indata, *uindata; /* Incoming data (uncompressed version) */
     q_header header; /* header data */
     q_question question;
-    int counter,place,count;
-    fd_set rx_set; /* Using select() because if its timeout option */
-    int maxd;      /* select() */
-    struct timeval tv;  /* select() */
-    int n; /* Select() return value */
+    int place,count;
     int nrd = 0; /* whether recursion is desired or not 1: No recursion;
                    0: recursion desired */
     int id;
@@ -164,33 +178,53 @@ main(int argc, char **argv) {
     int length;
 
     /* Determine what the query string is */
-    if(argc < 2 || argc > 6)
+    verbose_mode = 0;
+    timeout = 31;
+    server_address = "127.0.0.3";
+    argc--;
+    argv++;
+    while(argc > 0) {
+        if(argv[0] [0]== '-' && argv[0][1] == 'v')
+            verbose_mode = 1;
+        else if(argv[0][0] == '-' && argv[0][1] == 'n')
+            nrd = 1;
+        else if(argv[0][0] == '-' && argv[0][1] == 'p') {
+            if(argc < 2) {
+                harderror(L_USAGE);
+                }
+            argc--;
+            argv++;
+            dns_port = atoi(*argv);
+            if(dns_port < 1 || dns_port > 32000) {
+                    harderror("Wrong port number\n");
+                }
+            }
+        else if(argv[0][0] == '-' && argv[0][1] == 't') {
+            if(argc < 2) {
+                harderror(L_USAGE);
+                }
+            argc--;
+            argv++;
+            timeout = atoi(*argv);
+            if(timeout < 1)
+               harderror( L_INVALID_TIMEOUT);
+            verbose_mode = 2;
+            }
+        else if (argv[0][0] != '-')
+            break;
+        else
+            harderror(L_USAGE);
+        argc--;
+        argv++;
+        }
+
+    /* we expect now the query string and possibly the server */
+    if(argc<1)
         harderror(L_USAGE);
 
-    if(argc >= 3 && *argv[1] == '-' && *(argv[1] + 1) == 'n')
-        nrd = 1;
-
-    /* See if we are in verbose mode or not */
-    verbose_mode = 0;
-    if(argc >= 3 + nrd && *argv[1 + nrd] == '-' && *(argv[1 + nrd] + 1) == 'v')
-        verbose_mode = 1;
-
-    /* See if we should allow a user defined timeout or not */
-    timeout = 31;
-    if(argc >= 4 + nrd && *argv[1 + nrd] == '-' && *(argv[1 + nrd] + 1) == 
-       't') {
-         timeout = atoi(argv[2 + nrd]);
-	 verbose_mode = 2;
-	 }
-
-    if(timeout < 1) 
-        harderror(L_INVALID_TIMEOUT);
-
     /* Determine what IP address to connect to */
-    if(argc == 3 + verbose_mode + nrd)
-        server_address = argv[2 + verbose_mode + nrd];
-    else
-        server_address = "127.0.0.3";
+    if(argc>1)
+        server_address = argv[1];
 
     if((indata = js_create(512,1)) == 0)
        harderror(L_JS_CREATE_INDATA);
@@ -200,24 +234,49 @@ main(int argc, char **argv) {
        harderror(L_JS_CREATE_OUTDATA);
     if((question.qname = js_create(512,1)) == 0)
        harderror(L_JS_CREATE_QNAME);
-    /* Create a TCP socket */
-    if((s = socket(PF_INET,SOCK_STREAM,0)) == -1)
-        harderror(L_SOCKET);
 
     /* Create a socket address */
     memset(&server,0,sizeof(server));
+#if defined IPV6 && ! defined __CYGWIN__
+    if ( strchr(server_address,':') != NULL) {
+        /* assume that we have an IPv6 address */
+
+        /* Create a TCP socket */
+        if((s = socket(PF_INET6,SOCK_STREAM,0)) == -1)
+            harderror(L_SOCKET);
+
+        server.sin6_family = AF_INET6;
+        server.sin6_port = htons(dns_port);
+        if( inet_pton(AF_INET6, server_address, &server.sin6_addr) < 1) {
+            harderror(L_MAL_IP);
+            }
+        len_inet = sizeof(struct sockaddr_in6);
+        }
+    else {
+        if((s = socket(PF_INET,SOCK_STREAM,0)) == -1)
+            harderror(L_SOCKET);
+        ((struct sockaddr_in*)&server)->sin_family = AF_INET;
+        ((struct sockaddr_in*)&server)->sin_port = htons(dns_port);
+        if((((struct sockaddr_in*)&server)->sin_addr.s_addr = inet_addr(server_address)) == INADDR_NONE)
+            harderror(L_MAL_IP);
+        len_inet = sizeof(struct sockaddr_in);
+        }
+#else
+    if((s = socket(PF_INET,SOCK_STREAM,0)) == -1)
+        harderror(L_SOCKET);
+
     server.sin_family = AF_INET;
     /* When debugging MaraDNS on systems where I do not have root, it is
        useful to be able to contact a MaraDNS server running on a high
        port number.  However, DNS_PORT should be 53 otherwise (defined
        in MaraDNS.h) */
-    server.sin_port = htons(DNS_PORT);
+    server.sin_port = htons(dns_port);
     if((server.sin_addr.s_addr = inet_addr(server_address)) == INADDR_NONE)
         harderror(L_MAL_IP);
+    len_inet = sizeof(struct sockaddr_in);
+#endif
 
-    len_inet = sizeof(server);
-
-    id = gen_id(argv[1 + verbose_mode]);
+    id = gen_id(argv[0]);
     /* Format a DNS request */
     /* DNS header */
     header.id = id;
@@ -237,15 +296,17 @@ main(int argc, char **argv) {
     /* Create a DNS question , and put it in raw UDP format */
     /* DNS question -> looking up an A record... */
     question.qclass = 1; /* ...on the internet */
-    if(js_qstr2js(question.qname,argv[1 + verbose_mode + nrd]) == JS_ERROR)
+    if(js_qstr2js(question.qname,argv[0]) == JS_ERROR)
         harderror(L_INVALID_Q); /* Invalid query */
 
     /* Make sure that the csv1-compatible output hashes all non-RR
        output */
     if(verbose_mode != 1)
         printf("# ");
-
-    printf("%s%s%s",L_QUERYING,server_address,L_NEWLINE); /* Querying the server with the IP... */
+    if(dns_port==DNS_PORT)
+        printf("%s%s%s",L_QUERYING,server_address,L_NEWLINE); /* Querying the server with the IP... */
+    else
+        printf("%s%s#%d%s",L_QUERYING,server_address, dns_port,L_NEWLINE); /* Querying the server with the IP... */
 
     /* Make 'Aexample.com.' raw UDP data */
 
@@ -260,48 +321,46 @@ main(int argc, char **argv) {
 
     do {
         /* Connect to the zone server */
-	if(connect(s,(struct sockaddr *)&server,len_inet) == -1)
-	/* Determine how long it is */
-	printf("DEBUG: outdata->unit_count %d\n",outdata->unit_count);
-	len[0] = (outdata->unit_count & 0xff00) >> 8;
-	len[1] = outdata->unit_count & 0xff;
+        if(connect(s,(struct sockaddr *)&server,len_inet) == -1)
+        /* Determine how long it is */
+        printf("DEBUG: outdata->unit_count %d\n",outdata->unit_count);
+        len[0] = (outdata->unit_count & 0xff00) >> 8;
+        len[1] = outdata->unit_count & 0xff;
 
         /* Send out how long the request is */
-	if(write(s,len,2) == -1) {
-	    harderror("XXX No error message yet");
-	    }
+        if(write(s,len,2) == -1) {
+            harderror("XXX No error message yet 271");
+            }
 
         /* Send out a DNS request */
         if(write(s,outdata->string,outdata->unit_count) == -1)
-           harderror("XXX No error message yet");
+           harderror("XXX No error message yet 276");
 
         /* Wait for a reply from the DNS server */
         recv(s,len,2,MSG_WAITALL);
-	length = ((len[0] << 8) & 0xff00) | (len[1] & 0xff);
+        length = ((len[0] << 8) & 0xff00) | (len[1] & 0xff);
         if(length < 12)
-	    harderror("XXX No error message yet");
+            harderror("XXX No error message yet 282");
         if((count = recv(s,indata->string,length,MSG_WAITALL)) < 0)
             harderror(L_DNS_R_ERROR); /* Problem getting DNS server response */
 
         if(count != length)
-	     harderror("XXX No error message yet");
+             harderror("Length of reply not same as stated length");
 
         indata->unit_count = count;
 
         /* Get the ID from the remote response */
         if(count > 2) {
-	   id = (*(indata->string) << 8) | *(indata->string + 1);
-	   }
+           id = (*(indata->string) << 8) | *(indata->string + 1);
+           }
         else {
-	   id = header.id + 1;
-	   }
-
-        printf("DEBUG ID: %d header.id: %d\n",id,header.id);
+           id = header.id + 1;
+           }
 
         attempts--;
 
-	/* If the id doesn't match, get a reply again */
-	} while(id != header.id && attempts > 0); 
+        /* If the id doesn't match, get a reply again */
+        } while(id != header.id && attempts > 0);
 
     decomp_init(0);
     decompress_data(indata,uindata);
@@ -317,6 +376,7 @@ main(int argc, char **argv) {
     else
         csv1_compatible_output(uindata,question.qname);
 
+    return 0;
     }
 
 /* Verbose form of askmara output
@@ -434,9 +494,9 @@ int out_answer(js_string *uindata,int *place) {
             printf("%s%s",L_NEWLINE,L_S_RNAME); /* SOA rname */
             escape_stdout(soa.rname);
             printf("%s%s%u%s",L_NEWLINE,L_S_SERIAL,soa.serial,L_NEWLINE);
-            printf("%s%d%s",L_S_REFRESH,soa.refresh,L_NEWLINE);
-            printf("%s%d%s",L_S_RETRY,soa.retry,L_NEWLINE);
-            printf("%s%d%s",L_S_EXPIRE,soa.expire,L_NEWLINE);
+            printf("%s%d%s",L_S_REFRESH,(int)soa.refresh,L_NEWLINE);
+            printf("%s%d%s",L_S_RETRY,(int)soa.retry,L_NEWLINE);
+            printf("%s%d%s",L_S_EXPIRE,(int)soa.expire,L_NEWLINE);
             printf("%s%u%s",L_S_MINIMUM,soa.minimum,L_NEWLINE);
             }
         else if(rr_hdr.type == RR_MX) {
@@ -468,7 +528,24 @@ int out_answer(js_string *uindata,int *place) {
                    L_DOT,*(uindata->string + *place + 2),
                    L_DOT,*(uindata->string + *place + 3),L_NEWLINE,L_NEWLINE);
             }
-        else if(rr_hdr.type = RR_TXT) {
+        else if(rr_hdr.type == RR_AAAA) {
+            unsigned short *p;
+            if(uindata->unit_count < *place + 16)
+                return JS_ERROR;
+            /* Display the IP of the data */
+            /* Display the IP of the data */
+            p = (unsigned short*)(uindata->string + *place);
+            printf("%s%d%x:%x:%x:%x:%x:%x:%x:%x%s%s",L_IP,*(uindata->string + *place),
+                       htons(*(p + 0 )),
+                       htons(*(p + 1)),
+                       htons(*(p + 2)),
+                       htons(*(p + 3)),
+                       htons(*(p + 4)),
+                       htons(*(p + 5)),
+                       htons(*(p + 6)),
+                       htons(*(p + 7)),L_NEWLINE,L_NEWLINE);
+            }
+        else if(rr_hdr.type == RR_TXT) {
             printf("%s",L_TXT); /* Text String */
             read_txt(uindata,mx.exchange,*place);
             escape_stdout(mx.exchange);
@@ -479,6 +556,8 @@ int out_answer(js_string *uindata,int *place) {
 
         *place += rr_hdr.rdlength; /* To do: read the RD data itself for
                                       all MaraDNS supported RRs */
+
+        return 0;
         }
 
 /* csv1-compatible form of askmara output
@@ -508,10 +587,10 @@ int csv1_compatible_output(js_string *uindata, js_string *query) {
         if(count < 0) /* Error handling */
             harderror(L_QERROR); /* Error reading question */
         printf("# Question: ");
-	if(hname_translate(question.qname,question.qtype) == JS_ERROR) 
-	    harderror(L_QERROR);
+        if(hname_translate(question.qname,question.qtype) == JS_ERROR)
+            harderror(L_QERROR);
         show_esc_stdout(question.qname);
-	printf("\n");
+        printf("\n");
         place += count;
         }
 
@@ -585,6 +664,7 @@ int csv1_answer(js_string *uindata, int *place, js_string *query) {
     escape_stdout(rr_hdr.name);
     /* We also show the type if this is an unsupported record type */
     if(rr_hdr.type != RR_A &&
+       rr_hdr.type != RR_AAAA &&
        rr_hdr.type != RR_NS &&
        rr_hdr.type != RR_CNAME &&
        rr_hdr.type != RR_SOA &&
@@ -598,15 +678,15 @@ int csv1_answer(js_string *uindata, int *place, js_string *query) {
             harderror(L_READSOA);
         hname_translate(soa.mname,RR_SOA);
         hname_translate(soa.rname,RR_MX);
-        printf("|",L_S_MNAME); /* SOA mname */
+        printf("|"); /* SOA mname */
         escape_stdout(soa.mname);
-        printf("|",L_NEWLINE,L_S_RNAME); /* SOA rname */
+        printf("|"); /* SOA rname */
         escape_stdout(soa.rname);
-        printf("|%u",soa.serial,L_NEWLINE);
-        printf("|%d",soa.refresh,L_NEWLINE);
-        printf("|%d",soa.retry,L_NEWLINE);
-        printf("|%d",soa.expire,L_NEWLINE);
-        printf("|%u\n",soa.minimum,L_NEWLINE);
+        printf("|%u",soa.serial);
+        printf("|%d",(int)soa.refresh);
+        printf("|%d",(int)soa.retry);
+        printf("|%d",(int)soa.expire);
+        printf("|%u\n",soa.minimum);
         }
     else if(rr_hdr.type == RR_MX) {
         if(uindata->unit_count < *place + 2)
@@ -614,7 +694,7 @@ int csv1_answer(js_string *uindata, int *place, js_string *query) {
         mx.preference = ((*(uindata->string + *place) & 0xff) << 8) |
                          (*(uindata->string + *place + 1) & 0xff);
         read_ns(uindata,mx.exchange,*place + 2);
-        printf("|%d",mx.preference,L_NEWLINE); /* MX pref. */
+        printf("|%d",mx.preference); /* MX pref. */
         hname_translate(mx.exchange,-2); /* Makes the char a pipe */
         escape_stdout(mx.exchange);
         printf("\n");
@@ -635,17 +715,36 @@ int csv1_answer(js_string *uindata, int *place, js_string *query) {
                L_DOT,*(uindata->string + *place + 2),
                L_DOT,*(uindata->string + *place + 3));
         }
-    else if(rr_hdr.type = RR_TXT) {
-        printf("|"); 
+    else if(rr_hdr.type == RR_AAAA) {
+        unsigned short *p;
+        if(uindata->unit_count < *place + 16)
+            return JS_ERROR;
+        /* Display the IP of the data */
+        /* Display the IP of the data */
+        p = (unsigned short*)(uindata->string + *place);
+        printf("|%x:%x:%x:%x:%x:%x:%x:%x\n",
+                   htons(*(p + 0 )),
+                   htons(*(p + 1)),
+                   htons(*(p + 2)),
+                   htons(*(p + 3)),
+                   htons(*(p + 4)),
+                   htons(*(p + 5)),
+                   htons(*(p + 6)),
+                   htons(*(p + 7)));
+        }
+    else if(rr_hdr.type == RR_TXT) {
+        printf("|");
         read_txt(uindata,mx.exchange,*place);
         escape_stdout(mx.exchange);
         printf("\n");
         }
     else {
         printf("|\n");
-	}
+        }
 
     *place += rr_hdr.rdlength; /* To do: read the RD data itself for
                                   all MaraDNS supported RRs */
+
+    return 0;
     }
 
